#include <algorithm>
#include <fstream>
#include <stdlib.h>
#include "bench.hpp"
#include "cpp-vstack.hpp"
#include "cpp-vprint.hpp"
#include "object.hpp"

int main(int argc, char** argv) {
	srand(20171025);

	stack s;
	REPEAT_TIMED( "push_int",
		s.push( std::make_unique<integer>( rand() ) );
	)

	stack t;
	TIMED( "copy_int", 
		t = s;
	)

	TIMED( "clear_int", 
		s.clear();
	)

	integer max;
	REPEAT_TIMED( "pop_int",
		max = std::max( max, t.pop()->as<integer>() );
	)

	stack s2;
	REPEAT_TIMED( "push_bool_char",
		s2.push( std::make_unique<pair>( std::make_unique<boolean>( rand() & 0x1 ), 
			std::make_unique<character>( rand() & 0x7F ) ) );
	)

	stack t2;
	TIMED( "copy_bool_char", 
		t2 = s2;
	)

	TIMED( "clear_bool_char", 
		s2.clear();
	)

	auto max2 = std::make_unique<pair>( std::make_unique<boolean>(false), 
		std::make_unique<character>('\0') );
	REPEAT_TIMED( "pop_bool_char",
		std::unique_ptr<pair> x = as_ptr<pair>( t2.pop() );
		if ( *x > *max2 ) { max2 = std::move(x); }
	)

	std::ofstream out{"cpp-vout.txt"};
	REPEAT_TIMED( "print_int",
		print( out, integer{rand()}, c_string{":"}, integer{rand()}, c_string{"\n"} );
	)

	REPEAT_TIMED( "print_pair", 
		print( out, pair{ std::make_unique<boolean>( rand() & 0x1 ), 
			std::make_unique<character>( rand() & 0x7F ) }, c_string{":"}, 
			pair{ std::make_unique<boolean>( rand() & 0x1 ), 
			std::make_unique<character>( rand() & 0x7F ) }, c_string{"\n"} );
	)
}
